import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hub/backend/schema/enums/enums.dart';

import 'package:hub/flutter_flow/custom_functions.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/flutter_flow/nav/nav.dart';
import 'package:hub/shared/components/molecules/menu/index.dart';
import 'package:hub/shared/components/molecules/modules/index.dart';
import 'package:hub/shared/extensions/dialog_extensions.dart';
import 'package:hub/shared/services/authentication/authentication_service.dart';

class MenuViewEvent {}

class MenuViewState {
  final List<MenuItem?> menuEntries;
  final bool isGrid;

  MenuViewState({this.menuEntries = const [], this.isGrid = false});

  MenuViewState copyWith({List<MenuItem?>? menuEntries, bool? isGrid}) {
    return MenuViewState(
      menuEntries: menuEntries ?? this.menuEntries,
      isGrid: isGrid ?? this.isGrid,
    );
  }
}

class MenuViewBloc extends Bloc<MenuViewEvent, MenuViewState> {
  final MenuView style;
  final EnumMenuItem item;
  final bool expandable;
  final List<MenuEntry> menuOptions;
  late StreamSubscription<void> _completer;


  MenuViewBloc({required this.style, required this.item, required this.expandable, required this.menuOptions}) : super(MenuViewState()) {
    on<MenuViewEvent>((event, emit) async {
      await LicenseRemoteDataSourceImpl().waitForSaveCompletion();
      final entries = await generateMenuEntries();
      emit(state.copyWith(menuEntries: entries));
    });

    _completer = LicenseRemoteDataSourceImpl().streamCompleterController.listen((_) {
      add(MenuViewEvent());
    });
  }

  @override
  Future<void> close() {
    _completer.cancel();
    return super.close();
  }

  Future<MenuItem?> addMenuEntry(List<MenuItem?> entries, IconData icon, String text, Function() action) async {
    entries.add(
      item == EnumMenuItem.button
          ? MenuButtonWidget(icon: icon, action: action, title: text)
          : item == EnumMenuItem.card
              ? MenuCardItem(icon: icon, action: action, title: text)
              : item == EnumMenuItem.tile
                  ? MenuCardItem(icon: icon, action: action, title: text)
                  : null,
    );
    return null;
  }

  Future<List<MenuItem?>> generateMenuEntries() async {
    List<MenuItem?> entries = [];
    try {
      for (var opt in menuOptions) {
        String? licenseValue = await LicenseRepositoryImpl().g(opt.value);
        if (licenseValue != null) {
          Map<String, String> licenseMap = await stringToMap(licenseValue);
          final String display = licenseMap['display'] ?? 'INVISIVEL';
          final String startDate = licenseMap['startDate'] ?? '';
          final String expirationDate = licenseMap['expirationDate'] ?? '';
          final bool isStarted = await processStartDate(startDate, opt);
          final bool isExpired = await processExpirationDate(expirationDate, opt);
          if (isStarted && !isExpired) {
            await processDisplay(display, opt, entries);
          }
        }
      }
    } catch (e, s) {
      log('Error generating menu entries: $e', stackTrace:  s);

    }
    return entries;
  }

  Future<void> processDisplay(String display, MenuEntry opt, List<MenuItem?> entries) async {
    try {
      switch (display) {
        case 'VISIVEL':
          if(opt.value == 'FRE-HUB-LOGOUT')
            await addMenuEntry(entries, opt.icon, opt.name, () async {
              await AuthenticationService.signOut(navigatorKey.currentContext!);
            });
          else
          await addMenuEntry(entries, opt.icon, opt.name, () async {
            await nav(opt.route);
          });
          break;
        case 'DESABILITADO':
          await addMenuEntry(entries, opt.icon, opt.name, () async {
            await DialogUnavailable.unavailableFeature(navigatorKey.currentContext!);
          });
          break;
        case 'INVISIVEL':
        default:
          break;
      }
    } catch (e) {
      log('Error processing display for module ${opt.value}: $e');
    }
  }

  Future<bool> processStartDate(String startDate, MenuEntry opt) async {
    try {
      if (startDate.isEmpty) return true;
      final DateTime? start = DateTime.tryParse(startDate);
      if (start != null) {
        return DateTime.now().isAfter(start);
      }
    } catch (e) {
      log('Error processing start date for module ${opt.value}: $e');
    }
    return false;
  }

  Future<bool> processExpirationDate(String expirationDate, MenuEntry opt) async {
    try {
      if (expirationDate.isEmpty) return false;
      final DateTime? expiration = DateTime.tryParse(expirationDate);
      if (expiration != null) {
        return DateTime.now().isAfter(expiration);
      }
    } catch (e) {
      log('Error processing expiration date for module ${opt.value}: $e');
    }
    return false;
  }

  Future nav(String link) async {
    navigatorKey.currentContext!.push(link, extra: <String, dynamic>{
      kTransitionInfoKey: const TransitionInfo(
        hasTransition: false,
        transitionType: PageTransitionType.scale,
        alignment: Alignment.bottomCenter,
      ),
    });
  }
}
